package org.beigesoft.web.servlet;

/*
 * Copyright (c) 2017 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.HashMap;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Part;
import javax.servlet.RequestDispatcher;

import org.beigesoft.log.ILogger;
import org.beigesoft.service.ISrvI18n;
import org.beigesoft.web.service.UtlJsp;
import org.beigesoft.web.model.HttpRequestData;
import org.beigesoft.handler.IHandlerRequest;
import org.beigesoft.factory.IFactoryAppBeans;
import org.beigesoft.exception.ExceptionWithCode;

/**
 * <p>Generic servlet that upload single file in multipath way with name
 * from parameter named <b>paramNameFileToUpload</b>, then add pass attributes
 * "fileToUploadInputStream"-"[InputStream]" and
 * "fileToUploadName"-"[file name]" in "HttpRequestData" and delegate this
 * requestData to a IHandlerRequest that can implements any business logic
 * to handle this. fileToUploadInputStream should be closed by delegate.
 * Delete action required parameter <b>nameFieldPath</b> to retrieve
 * path with reflection to delete uploaded file.
 * Entity maybe 1-st level  or owned.
 * File may be already uploaded, i.e. only file path. File maybe NULL.
 * if parameter "paramNameFileToUpload" not null, then file will be uploaded,
 * otherwise it's uploaded path or NULL.
 * Of course, servlet must be configured with "multipart-config",
 * and so does WEB-form.
 * Handler is usually "HandlerEntityRequest"
 * that fills all others entity's fields.
 * There are two kind of entity with file:
 * 1. an attachment (e.g. email)
 * 2. image/file on HTML page (e.g. WEB-Store's item specifics "Image")
 * WEB-form must POST parameters:
 * <ul>
 * <li><b>Inserts/updates with file/uploaded path:</b></li>
 * <li>paramNameFileToUpload - parameter file to upload
 *   or NULL for already  uploaded path.</li>
 * <li>fieldNameFileName - entity's field name that holds file name,
 *  e.g. attachment name, maybe NULL</li>
 * <li>fieldNameFilePath - entity's field name that holds file path,
 *  it's NULL for already uploaded URL</li>
 * <li>fieldNameFileUrl - entity's field name that holds file URL,
 *  it's used to make file URL in HTML page from uploading file path,
 *  maybe NULL (e.g. for mail attachment, or uploaded URL).</li>
 * <li><b>Deletes with file:</b></li>
 * <li>fieldNameFilePath - entity's field name that holds file path,
 *  not NULL, if. file not found, then it does nothing.</li>
 * </ul>
 * In case of delete, NULL or uploaded URL this servlet acts like ordinal
 * WHandlerAndJsp,  i.e. without file uploading, so use WHandlerAndJsp
 * for action "delete".
 * Some entities require "update" action to upload file, e.g. GoodsSpecifics
 * with Specifics of type "Image" or "File", and it also requires designates
 * processor depend of type, so pass parameter "nmActEF" to editEntityF(fol).jsp
 * for farther assigning processor name in holder entity's processors
 * names (IHolderForClassByName<String>).
 * </p>
 *
 * @author Yury Demidenko
 */
@SuppressWarnings("serial")
public class WUploadSingle extends HttpServlet {

  /**
   * <p>App beans factort.</p>
   **/
  private IFactoryAppBeans factoryAppBeans;

  /**
   * <p>Folder for redirected JSP, e.g. "JSP WEB-INF/jsp/".
   * Settled through init params.</p>
   **/
  private String dirJsp;

  /**
   * <p>Request handler name, usually of type HandlerEntityRequest.</p>
   **/
  private String handlerName;

  @Override
  public final void init() throws ServletException {
    this.dirJsp = getInitParameter("dirJsp");
    this.handlerName = getInitParameter("handlerName");
    try {
      this.factoryAppBeans = (IFactoryAppBeans) getServletContext()
        .getAttribute("IFactoryAppBeans");
    } catch (Exception e) {
      if (this.factoryAppBeans != null) {
        ILogger logger = null;
        try {
          logger = (ILogger) this.factoryAppBeans.lazyGet("ILogger");
        } catch (Exception e1) {
          e1.printStackTrace();
        }
        if (logger != null) {
          logger.error(null, getClass(), "INIT", e);
        }
      }
      throw new ServletException(e);
    }
  }

  @Override
  public final void doPost(final HttpServletRequest pReq,
    final HttpServletResponse pResp) throws ServletException, IOException {
    pReq.setCharacterEncoding("UTF-8");
    pResp.setCharacterEncoding("UTF-8");
    try {
      HashMap<String, Object> reqVars = new HashMap<String, Object>();
      HttpRequestData requestData = new HttpRequestData(pReq, pResp);
      requestData.setAttribute("reqVars", reqVars);
      IHandlerRequest hndlI18nRequest = (IHandlerRequest) this
        .factoryAppBeans.lazyGet("hndlI18nRequest");
      hndlI18nRequest.handle(reqVars, requestData);
      // e.g. Eattachment.itsPath
      String fileParam = pReq.getParameter("paramNameFileToUpload");
      if (fileParam != null) { // create
        Part filePart = pReq.getPart(fileParam);
        String fileName = getSubmittedFileName(filePart);
        InputStream fileToUploadIs = filePart.getInputStream();
        requestData.setAttribute("fileToUploadInputStream", fileToUploadIs);
        requestData.setAttribute("fileToUploadName", fileName);
      } // else - delete file (if not NULL)
      // or insert entity with already uploaded file path or NULL file
      IHandlerRequest srvHandleRequest = (IHandlerRequest) this
        .factoryAppBeans.lazyGet(this.handlerName);
      srvHandleRequest.handle(reqVars, requestData);
      ISrvI18n srvI18n = (ISrvI18n) this.factoryAppBeans.lazyGet("ISrvI18n");
      UtlJsp utlJsp = (UtlJsp) this.factoryAppBeans.lazyGet("UtlJsp");
      pReq.setAttribute("srvI18n", srvI18n);
      pReq.setAttribute("utlJsp", utlJsp);
      String nmRnd = pReq.getParameter("nmRnd");
      String path = dirJsp + nmRnd + ".jsp";
      RequestDispatcher rd = getServletContext().getRequestDispatcher(path);
      rd.include(pReq, pResp);
    } catch (Exception e) {
      if (this.factoryAppBeans != null) {
        ILogger logger = null;
        try {
          logger = (ILogger) this.factoryAppBeans.lazyGet("ILogger");
        } catch (Exception e1) {
          e1.printStackTrace();
        }
        if (logger != null) {
          logger.error(null, getClass(), "WORK", e);
        } else {
          e.printStackTrace();
        }
      } else {
        e.printStackTrace();
      }
      if (e instanceof ExceptionWithCode) {
        pReq.setAttribute("error_code",
          ((ExceptionWithCode) e).getCode());
        pReq.setAttribute("short_message",
          ((ExceptionWithCode) e).getShortMessage());
      } else {
        pReq.setAttribute("error_code",
          HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
      }
      pReq.setAttribute("javax.servlet.error.status_code",
        HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
      pReq.setAttribute("javax.servlet.error.exception", e);
      pReq.setAttribute("javax.servlet.error.request_uri",
        pReq.getRequestURI());
      pReq.setAttribute("javax.servlet.error.servlet_name", this.getClass()
        .getCanonicalName());
      pResp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
    }
  }

  /**
   * <p>Retrieve file name from part.</p>
   * @param pPart file data
   * @return file name
   * @throws Exception - an exception
   **/
  public final String getSubmittedFileName(
    final Part pPart) throws Exception {
    for (String cd : pPart.getHeader("content-disposition").split(";")) {
      if (cd.trim().startsWith("filename")) {
        String fileName = cd.substring(cd.indexOf('=') + 1)
          .trim().replace("\"", "");
        return fileName.substring(fileName.lastIndexOf('/') + 1)
          .substring(fileName.lastIndexOf('\\') + 1); // MSIE fix.
      }
    }
    throw new ExceptionWithCode(ExceptionWithCode.SOMETHING_WRONG,
      "Can't retrieve file name");
  }

  //Simple getters and setters:
  /**
   * <p>Geter for factoryAppBeans.</p>
   * @return IFactoryAppBeans
   **/
  public final IFactoryAppBeans getFactoryAppBeans() {
    return this.factoryAppBeans;
  }

  /**
   * <p>Setter for factoryAppBeans.</p>
   * @param pFactoryAppBeans reference
   **/
  public final void setFactoryAppBeans(
    final IFactoryAppBeans pFactoryAppBeans) {
    this.factoryAppBeans = pFactoryAppBeans;
  }

  /**
   * <p>Geter for dirJsp.</p>
   * @return String
   **/
  public final String getDirJsp() {
    return this.dirJsp;
  }

  /**
   * <p>Setter for dirJsp.</p>
   * @param pDirJsp reference
   **/
  public final void setDirJsp(final String pDirJsp) {
    this.dirJsp = pDirJsp;
  }

  /**
   * <p>Getter for handlerName.</p>
   * @return String
   **/
  public final String getHandlerName() {
    return this.handlerName;
  }

  /**
   * <p>Setter for handlerName.</p>
   * @param pHandlerName reference
   **/
  public final void setHandlerName(final String pHandlerName) {
    this.handlerName = pHandlerName;
  }
}
